package java.text;

import java.util.ArrayList;

final class MergeCollation
{
    ArrayList<PatternEntry> patterns;
    private transient PatternEntry saveEntry;
    private transient PatternEntry lastEntry;
    private transient StringBuffer excess;
    private transient byte[] statusArray;
    private final byte BITARRAYMASK = 1;
    private final int BYTEPOWER = 3;
    private final int BYTEMASK = 7;
    
    public MergeCollation(String pattern) throws ParseException {
        this.patterns = new ArrayList();
        this.saveEntry = null;
        this.lastEntry = null;
        this.excess = new StringBuffer();
        this.statusArray = new byte[8192];
        for (int i = 0; i < this.statusArray.length; ++i) {
            this.statusArray[i] = 0;
        }
        this.setPattern(pattern);
    }
    
    public String getPattern() {
        return this.getPattern(true);
    }
    
    public String getPattern(boolean b) {
        StringBuffer sb = new StringBuffer();
        Object o = null;
        int i;
        for (i = 0; i < this.patterns.size(); ++i) {
            PatternEntry e = (PatternEntry)this.patterns.get(i);
            if (e.extension.length() != 0) {
                if (o == null) {
                    o = new ArrayList();
                }
                o.add(e);
            }
            else {
                if (o != null) {
                    PatternEntry lastWithNoExtension = this.findLastWithNoExtension(i - 1);
                    for (int j = o.size() - 1; j >= 0; --j) {
                        ((PatternEntry)o.get(j)).addToBuffer(sb, false, b, lastWithNoExtension);
                    }
                    o = null;
                }
                e.addToBuffer(sb, false, b, null);
            }
        }
        if (o != null) {
            PatternEntry lastWithNoExtension2 = this.findLastWithNoExtension(i - 1);
            for (int k = o.size() - 1; k >= 0; --k) {
                ((PatternEntry)o.get(k)).addToBuffer(sb, false, b, lastWithNoExtension2);
            }
        }
        return sb.toString();
    }
    
    private final PatternEntry findLastWithNoExtension(int i) {
        --i;
        while (i >= 0) {
            PatternEntry patternEntry = (PatternEntry)this.patterns.get(i);
            if (patternEntry.extension.length() == 0) {
                return patternEntry;
            }
            --i;
        }
        return null;
    }
    
    public String emitPattern() {
        return this.emitPattern(true);
    }
    
    public String emitPattern(boolean b) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < this.patterns.size(); ++i) {
            PatternEntry patternEntry = (PatternEntry)this.patterns.get(i);
            if (patternEntry != null) {
                patternEntry.addToBuffer(sb, true, b, null);
            }
        }
        return sb.toString();
    }
    
    public void setPattern(String s) throws ParseException {
        this.patterns.clear();
        this.addPattern(s);
    }
    
    public void addPattern(String s) throws ParseException {
        if (s == null) {
            return;
        }
        PatternEntry.Parser parser = new PatternEntry.Parser(s);
        for (PatternEntry patternEntry = parser.next(); patternEntry != null; patternEntry = parser.next()) {
            this.fixEntry(patternEntry);
        }
        return;
    }
    
    public int getCount() {
        return this.patterns.size();
    }
    
    public PatternEntry getItemAt(int index) {
        return (PatternEntry)this.patterns.get(index);
    }
    
    private final void fixEntry(PatternEntry obj) throws ParseException {
        if (this.lastEntry != null && obj.chars.equals(this.lastEntry.chars) && obj.extension.equals(this.lastEntry.extension)) {
            if (obj.strength != 3 && obj.strength != -2) {
                throw new ParseException("The entries " + this.lastEntry + " and " + obj + " are adjacent in the rules, but have conflicting strengths: A character can't be unequal to itself.", -1);
            }
            return;
        }
        else {
            boolean n = true;
            if (obj.strength != -2) {
                int index = -1;
                if (obj.chars.length() == 1) {
                    int char1 = obj.chars.charAt(0);
                    int n2 = char1 >> 3;
                    int n3 = this.statusArray[n2];
                    int n4 = (byte)(1 << (char1 & '\u0007'));
                    if (n3 != 0 && (n3 & n4) != 0x0) {
                        index = this.patterns.lastIndexOf(obj);
                    }
                    else {
                        this.statusArray[n2] = (byte)(n3 | n4);
                    }
                }
                else {
                    index = this.patterns.lastIndexOf(obj);
                }
                if (index != -1) {
                    this.patterns.remove(index);
                }
                this.excess.setLength(0);
                int lastEntry = this.findLastEntry(this.lastEntry, this.excess);
                if (this.excess.length() != 0) {
                    obj.extension = this.excess + obj.extension;
                    if (lastEntry != this.patterns.size()) {
                        this.lastEntry = this.saveEntry;
                        n = false;
                    }
                }
                if (lastEntry == this.patterns.size()) {
                    this.patterns.add(obj);
                    this.saveEntry = obj;
                }
                else {
                    this.patterns.add(lastEntry, obj);
                }
            }
            if (!(n == 0)) {
                this.lastEntry = obj;
            }
            return;
        }
    }
    
    private final int findLastEntry(PatternEntry patternEntry, StringBuffer sb) throws ParseException {
        if (patternEntry == null) {
            return 0;
        }
        if (patternEntry.strength != -2) {
            int n = -1;
            if (patternEntry.chars.length() == 1) {
                if ((this.statusArray[patternEntry.chars.charAt(0) >> 3] & 1 << (patternEntry.chars.charAt(0) & '\u0007')) != 0x0) {
                    n = this.patterns.lastIndexOf(patternEntry);
                }
            }
            else {
                n = this.patterns.lastIndexOf(patternEntry);
            }
            if (n == -1) {
                throw new ParseException("couldn't find last entry: " + patternEntry, n);
            }
            return n + 1;
        }
        else {
            int i = this.patterns.size() - 1;
            while (i >= 0) {
                PatternEntry patternEntry2 = (PatternEntry)this.patterns.get(i);
                if (patternEntry2.chars.regionMatches(0, patternEntry.chars, 0, patternEntry2.chars.length())) {
                    sb.append(patternEntry.chars.substring(patternEntry2.chars.length(), patternEntry.chars.length()));
                    break;
                }
                --i;
            }
            if (i == -1) {
                throw new ParseException("couldn't find: " + patternEntry, i);
            }
            return i + 1;
        }
    }
}
